from PyQt5 import QtWidgets, QtGui, QtCore
from pymeasure.display.widgets import TabWidget

from ..classes.hall_sensor import HallSensor

class LiveTab(TabWidget, QtWidgets.QWidget):
    def __init__(self, name, parent=None):
        super().__init__(parent)
        self.name = name
        self.hall_sensor = HallSensor()

        font = QtGui.QFont()
        font.setPointSize(32)

        layout = QtWidgets.QGridLayout()

        self.sensor_value_label = QtWidgets.QLabel("0.0")
        layout.addWidget(self.sensor_value_label, 0, 1)
        self.sensor_value_label.setFont(font)

        self.zero_button = QtWidgets.QPushButton("Zero")
        layout.addWidget(self.zero_button, 1, 0, 1, 2)
        self.zero_button.clicked.connect(self.zero_sensor_value)

        self.setLayout(layout)

        self.sensor_timer = QtCore.QTimer()
        self.sensor_timer.timeout.connect(self.update_sensor_value)
        self.sensor_timer.start(100)  # Refresh every 0.1 seconds

        self.thread_pool = QtCore.QThreadPool()

        self.tabBarClicked.connect(self.tab_activated)
        self.tabBarDoubleClicked.connect(self.tab_activated)

    def tab_activated(self, index):
        if index == self.currentIndex():
            self.sensor_timer.start(100)  # Start updating sensor value
        else:
            self.sensor_timer.stop()  # Stop updating sensor value

    def update_sensor_value(self):
        # Create a worker object for reading the sensor value
        worker = SensorWorker(self.hall_sensor)
        worker.sensor_value_updated.connect(self.update_sensor_label)

        # Move the worker object to the thread pool and start the thread
        self.thread_pool.start(worker)

    def update_sensor_label(self, sensor_value):
        self.sensor_value_label.setText(str(sensor_value) + "mT")

    def zero_sensor_value(self):
        self.sensor_timer.stop()
        self.hall_sensor.zeroing()
        self.sensor_timer.start(100)

class SensorWorker(QtCore.QObject):
    sensor_value_updated = QtCore.pyqtSignal(float)

    def __init__(self, hall_sensor):
        super().__init__()
        self.hall_sensor = hall_sensor

    @QtCore.pyqtSlot()
    def run(self):
        sensor_value = self.hall_sensor.read_mT()
        self.sensor_value_updated.emit(sensor_value)
